# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement

import os

try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

from raxmon_cli.constants import CONFIG_PATH

__all___ = [
    'str_to_list',
    'str_to_dict',
    'instance_to_dict',
    'get_credentials',
    'read_json_from_file'
]


def str_to_list(string):
    if not string:
        return None

    return string.split(',')


def str_to_dict(string):
    if not string:
        return None

    result = {}
    split = string.split(',')

    for value in split:
        split2 = value.split('=')

        if len(split2) == 2:
            key, value = split2
            result[key] = value

    return result


def instance_to_dict(instance, keys, include_none=False):
    result = {}

    for key in keys:
        value = getattr(instance, key, None)

        if value is None and not include_none:
            continue

        result[key] = value
    return result


def get_config():
    keys = [['credentials', 'username', 'username'],
            ['credentials', 'api_key', 'api_key'],
            ['api', 'url', 'api_url'],
            ['auth_api', 'url', 'auth_url'],
            ['ssl', 'verify', 'ssl_verify']]

    result = {}

    result['username'] = os.getenv('RAXMON_USERNAME', None)
    result['api_key'] = os.getenv('RAXMON_API_KEY', None)
    result['api_url'] = os.getenv('RAXMON_API_URL', None)
    result['auth_url'] = os.getenv('RAXMON_AUTH_URL', None)
    result['ssl_verify'] = os.getenv('RAXMON_SSL_VERIFY', None)

    config = ConfigParser.ConfigParser()
    config.read(os.getenv('RAXMON_RAXRC') or CONFIG_PATH)

    for (config_section, config_key, key) in keys:
        if result[key]:
            # Already specified as an env variable
            continue

        try:
            value = config.get(config_section, config_key)
        except ConfigParser.Error:
            continue

        result[key] = value

    # convert "false" to False
    if result['ssl_verify']:
        result['ssl_verify'] = not (result['ssl_verify'].lower() == 'false')
    else:
        result['ssl_verify'] = True

    return result


def read_json_from_file(file_path):
    with open(file_path, 'r') as fp:
        content = fp.read()

    return content



# Generated by the Waxeye Parser Generator - version 0.8.0
# www.waxeye.org

from waxeye import Edge, State, FA, WaxeyeParser

class Parser (WaxeyeParser):
    start = 0
    eof_check = True
    automata = [FA("alarmDsl", [State([Edge(32, 1, False)], False),
            State([Edge(23, 1, False),
                Edge(32, 2, False)], False),
            State([Edge(10, 2, False),
                Edge(32, 3, False)], False),
            State([Edge(18, 3, False)], True)], FA.LEFT),
        FA("metricName", [State([Edge("m", 1, True)], False),
            State([Edge("e", 2, True)], False),
            State([Edge("t", 3, True)], False),
            State([Edge("r", 4, True)], False),
            State([Edge("i", 5, True)], False),
            State([Edge("c", 6, True)], False),
            State([Edge("[", 7, True)], False),
            State([Edge("\"", 8, True),
                Edge("\'", 12, True)], False),
            State([Edge(5, 9, False)], False),
            State([Edge("\"", 10, True)], False),
            State([Edge("]", 11, True)], False),
            State([], True),
            State([Edge(5, 13, False)], False),
            State([Edge("\'", 10, True)], False)], FA.LEFT),
        FA("dimension", [State([Edge("d", 1, True)], False),
            State([Edge("i", 2, True)], False),
            State([Edge("m", 3, True)], False),
            State([Edge("e", 4, True)], False),
            State([Edge("n", 5, True)], False),
            State([Edge("s", 6, True)], False),
            State([Edge("i", 7, True)], False),
            State([Edge("o", 8, True)], False),
            State([Edge("n", 9, True)], False),
            State([Edge("[", 10, True)], False),
            State([Edge("\"", 11, True),
                Edge("\'", 15, True)], False),
            State([Edge(5, 12, False)], False),
            State([Edge("\"", 13, True)], False),
            State([Edge("]", 14, True)], False),
            State([], True),
            State([Edge(5, 16, False)], False),
            State([Edge("\'", 13, True)], False)], FA.LEFT),
        FA("metricModifier", [State([Edge("r", 1, False)], False),
            State([Edge("a", 2, False)], False),
            State([Edge("t", 3, False)], False),
            State([Edge("e", 4, False)], False),
            State([], True)], FA.LEFT),
        FA("metric", [State([Edge(3, 1, False),
                Edge(1, 6, False)], False),
            State([Edge("(", 2, True)], False),
            State([Edge(32, 3, False)], False),
            State([Edge(1, 4, False)], False),
            State([Edge(32, 5, False)], False),
            State([Edge(")", 6, True)], False),
            State([], True)], FA.PRUNE),
        FA("name", [State([Edge(["-", "_", (97, 122)], 1, False),
                Edge(" ", 1, False),
                Edge([(48, 57)], 1, False)], False),
            State([Edge(["-", "_", (97, 122)], 1, False),
                Edge(" ", 1, False),
                Edge([(48, 57)], 1, False)], True)], FA.LEFT),
        FA("inExp", [State([Edge(4, 1, False),
                Edge(2, 4, False)], False),
            State([Edge(32, 2, False)], False),
            State([Edge(7, 3, False)], False),
            State([], True),
            State([Edge(32, 5, False)], False),
            State([Edge(22, 6, False)], False),
            State([Edge(32, 7, False)], False),
            State([Edge(13, 3, False)], False)], FA.PRUNE),
        FA("rhs", [State([Edge(21, 1, False),
                Edge(22, 4, False)], False),
            State([Edge(32, 2, False)], False),
            State([Edge(12, 3, False)], False),
            State([], True),
            State([Edge(32, 5, False)], False),
            State([Edge(13, 3, False)], False)], FA.LEFT),
        FA("condExp", [State([Edge(32, 1, False)], False),
            State([Edge("(", 2, True),
                Edge(6, 6, False)], False),
            State([Edge(32, 3, False)], False),
            State([Edge(6, 4, False)], False),
            State([Edge(32, 5, False)], False),
            State([Edge(")", 6, True)], False),
            State([Edge(32, 7, False)], False),
            State([], True)], FA.PRUNE),
        FA("conj", [State([Edge("&", 1, False),
                Edge("|", 3, False)], False),
            State([Edge("&", 2, False)], False),
            State([], True),
            State([Edge("|", 2, False)], False)], FA.LEFT),
        FA("ifStatement", [State([Edge("i", 1, True)], False),
            State([Edge("f", 2, True)], False),
            State([Edge(32, 3, False)], False),
            State([Edge("(", 4, True)], False),
            State([Edge(32, 5, False)], False),
            State([Edge(8, 6, False)], False),
            State([Edge(9, 7, False),
                Edge(")", 9, True)], False),
            State([Edge(32, 8, False)], False),
            State([Edge(8, 6, False)], False),
            State([Edge(32, 10, False)], False),
            State([Edge(11, 11, False)], False),
            State([Edge(32, 12, False)], False),
            State([], True)], FA.LEFT),
        FA("block", [State([Edge("{", 1, True)], False),
            State([Edge(32, 2, False)], False),
            State([Edge(18, 3, False)], False),
            State([Edge(32, 4, False)], False),
            State([Edge("}", 5, True)], False),
            State([], True)], FA.LEFT),
        FA("nValue", [State([Edge(16, 1, False),
                Edge(4, 1, False)], False),
            State([], True)], FA.LEFT),
        FA("tValue", [State([Edge(14, 1, False),
                Edge(4, 1, False)], False),
            State([], True)], FA.LEFT),
        FA("string", [State([Edge("\"", 1, True),
                Edge("\'", 6, True)], False),
            State([Edge("\\", 2, True),
                Edge(34, 3, False),
                Edge("\"", 5, True)], False),
            State([Edge(15, 1, False)], False),
            State([Edge(33, 4, False)], False),
            State([Edge(-1, 1, False)], False),
            State([], True),
            State([Edge("\\", 7, True),
                Edge(36, 8, False),
                Edge("\'", 5, True)], False),
            State([Edge(15, 6, False)], False),
            State([Edge(35, 9, False)], False),
            State([Edge(-1, 6, False)], False)], FA.LEFT),
        FA("escaped", [State([Edge("u", 1, False),
                Edge(["\"", "\'", "/", "\\", "b", "f", "n", "r", "t"], 5, False)], False),
            State([Edge([(48, 57), (65, 70), (97, 102)], 2, False)], False),
            State([Edge([(48, 57), (65, 70), (97, 102)], 3, False)], False),
            State([Edge([(48, 57), (65, 70), (97, 102)], 4, False)], False),
            State([Edge([(48, 57), (65, 70), (97, 102)], 5, False)], False),
            State([], True)], FA.PRUNE),
        FA("number", [State([Edge("-", 1, False),
                Edge("0", 2, False),
                Edge([(49, 57)], 8, False)], False),
            State([Edge("0", 2, False),
                Edge([(49, 57)], 8, False)], False),
            State([Edge(".", 3, False),
                Edge(["E", "e"], 5, False)], True),
            State([Edge([(48, 57)], 4, False)], False),
            State([Edge([(48, 57)], 4, False),
                Edge(["E", "e"], 5, False)], True),
            State([Edge(["+", "-"], 6, False),
                Edge([(48, 57)], 7, False)], False),
            State([Edge([(48, 57)], 7, False)], False),
            State([Edge([(48, 57)], 7, False)], True),
            State([Edge([(48, 57)], 8, False),
                Edge(".", 3, False),
                Edge(["E", "e"], 5, False)], True)], FA.LEFT),
        FA("wholeNumber", [State([Edge([(49, 57)], 1, False)], False),
            State([Edge([(48, 57)], 1, False)], True)], FA.LEFT),
        FA("return", [State([Edge("r", 1, False)], False),
            State([Edge("e", 2, False)], False),
            State([Edge("t", 3, False)], False),
            State([Edge("u", 4, False)], False),
            State([Edge("r", 5, False)], False),
            State([Edge("n", 6, False)], False),
            State([Edge(" ", 7, False)], False),
            State([Edge(32, 8, False)], False),
            State([Edge(19, 9, False)], False),
            State([Edge(32, 10, False)], False),
            State([Edge(",", 11, True)], True),
            State([Edge(32, 12, False)], False),
            State([Edge(20, 13, False)], False),
            State([Edge(32, 14, False)], False),
            State([], True)], FA.LEFT),
        FA("alertStates", [State([Edge("C", 1, False),
                Edge("O", 9, False),
                Edge("W", 10, False)], False),
            State([Edge("R", 2, False)], False),
            State([Edge("I", 3, False)], False),
            State([Edge("T", 4, False)], False),
            State([Edge("I", 5, False)], False),
            State([Edge("C", 6, False)], False),
            State([Edge("A", 7, False)], False),
            State([Edge("L", 8, False)], False),
            State([], True),
            State([Edge("K", 8, False)], False),
            State([Edge("A", 11, False)], False),
            State([Edge("R", 12, False)], False),
            State([Edge("N", 13, False)], False),
            State([Edge("I", 14, False)], False),
            State([Edge("N", 15, False)], False),
            State([Edge("G", 8, False)], False)], FA.LEFT),
        FA("alertStateReason", [State([Edge(14, 1, False)], False),
            State([], True)], FA.LEFT),
        FA("nOp", [State([Edge(">", 1, False),
                Edge("<", 4, False),
                Edge(">", 2, False),
                Edge("<", 2, False),
                Edge("=", 5, False),
                Edge("!", 6, False)], False),
            State([Edge("=", 2, False)], False),
            State([Edge(32, 3, False)], False),
            State([], True),
            State([Edge("=", 2, False)], False),
            State([Edge("=", 2, False)], False),
            State([Edge("=", 2, False)], False)], FA.LEFT),
        FA("tOp", [State([Edge("n", 1, False),
                Edge("r", 8, False),
                Edge("=", 12, False),
                Edge("!", 13, False)], False),
            State([Edge("r", 2, False)], False),
            State([Edge("e", 3, False)], False),
            State([Edge("g", 4, False)], False),
            State([Edge("e", 5, False)], False),
            State([Edge("x", 6, False)], False),
            State([Edge(32, 7, False)], False),
            State([], True),
            State([Edge("e", 9, False)], False),
            State([Edge("g", 10, False)], False),
            State([Edge("e", 11, False)], False),
            State([Edge("x", 6, False)], False),
            State([Edge("=", 6, False)], False),
            State([Edge("=", 6, False)], False)], FA.LEFT),
        FA("setStatement", [State([Edge(":", 1, True)], False),
            State([Edge("s", 2, True)], False),
            State([Edge("e", 3, True)], False),
            State([Edge("t", 4, True)], False),
            State([Edge(32, 5, False)], False),
            State([Edge(24, 6, False)], False),
            State([Edge(32, 7, False)], False),
            State([], True)], FA.LEFT),
        FA("inSetStatement", [State([Edge(26, 1, False),
                Edge(27, 1, False)], False),
            State([], True)], FA.PRUNE),
        FA("consistencyLevel", [State([Edge("O", 1, False),
                Edge("Q", 4, False),
                Edge("A", 9, False)], False),
            State([Edge("N", 2, False)], False),
            State([Edge("E", 3, False)], False),
            State([], True),
            State([Edge("U", 5, False)], False),
            State([Edge("O", 6, False)], False),
            State([Edge("R", 7, False)], False),
            State([Edge("U", 8, False)], False),
            State([Edge("M", 3, False)], False),
            State([Edge("L", 10, False)], False),
            State([Edge("L", 3, False)], False)], FA.LEFT),
        FA("setConsistencyLevel", [State([Edge("c", 1, True)], False),
            State([Edge("o", 2, True)], False),
            State([Edge("n", 3, True)], False),
            State([Edge("s", 4, True)], False),
            State([Edge("i", 5, True)], False),
            State([Edge("s", 6, True)], False),
            State([Edge("t", 7, True)], False),
            State([Edge("e", 8, True)], False),
            State([Edge("n", 9, True)], False),
            State([Edge("c", 10, True)], False),
            State([Edge("y", 11, True)], False),
            State([Edge("L", 12, True)], False),
            State([Edge("e", 13, True)], False),
            State([Edge("v", 14, True)], False),
            State([Edge("e", 15, True)], False),
            State([Edge("l", 16, True)], False),
            State([Edge(32, 17, False)], False),
            State([Edge("=", 18, True)], False),
            State([Edge(32, 19, False)], False),
            State([Edge(25, 20, False)], False),
            State([], True)], FA.LEFT),
        FA("setConsecutiveCount", [State([Edge("c", 1, True)], False),
            State([Edge("o", 2, True)], False),
            State([Edge("n", 3, True)], False),
            State([Edge("s", 4, True)], False),
            State([Edge("e", 5, True)], False),
            State([Edge("c", 6, True)], False),
            State([Edge("u", 7, True)], False),
            State([Edge("t", 8, True)], False),
            State([Edge("i", 9, True)], False),
            State([Edge("v", 10, True)], False),
            State([Edge("e", 11, True)], False),
            State([Edge("C", 12, True)], False),
            State([Edge("o", 13, True)], False),
            State([Edge("u", 14, True)], False),
            State([Edge("n", 15, True)], False),
            State([Edge("t", 16, True)], False),
            State([Edge(32, 17, False)], False),
            State([Edge("=", 18, True)], False),
            State([Edge(32, 19, False)], False),
            State([Edge(17, 20, False)], False),
            State([], True)], FA.LEFT),
        FA("sComment", [State([Edge("#", 1, False)], False),
            State([Edge(38, 2, False),
                Edge(30, 3, False),
                Edge(37, 3, False)], False),
            State([Edge(-1, 1, False)], False),
            State([], True)], FA.VOID),
        FA("mComment", [State([Edge("/", 1, False)], False),
            State([Edge("*", 2, False)], False),
            State([Edge(29, 2, False),
                Edge(39, 3, False),
                Edge("*", 4, False)], False),
            State([Edge(-1, 2, False)], False),
            State([Edge("/", 5, False)], False),
            State([], True)], FA.VOID),
        FA("endOfLine", [State([Edge("\r", 1, False),
                Edge("\n", 2, False),
                Edge("\r", 2, False)], False),
            State([Edge("\n", 2, False)], False),
            State([], True)], FA.VOID),
        FA("wsp", [State([Edge(["\t", " "], 0, False)], True)], FA.VOID),
        FA("ws", [State([Edge(30, 0, False),
                Edge(28, 0, False),
                Edge(29, 0, False),
                Edge(["\t", " "], 0, False)], True)], FA.VOID),
        FA("", [State([Edge("\"", 1, False)], False),
            State([], True)], FA.NEG),
        FA("", [State([Edge("\\", 1, False)], False),
            State([], True)], FA.NEG),
        FA("", [State([Edge("\'", 1, False)], False),
            State([], True)], FA.NEG),
        FA("", [State([Edge("\\", 1, False)], False),
            State([], True)], FA.NEG),
        FA("", [State([Edge(-1, 1, False)], False),
            State([], True)], FA.NEG),
        FA("", [State([Edge(30, 1, False)], False),
            State([], True)], FA.NEG),
        FA("", [State([Edge("*", 1, False)], False),
            State([Edge("/", 2, False)], False),
            State([], True)], FA.NEG)]

    def __init__(self):
        WaxeyeParser.__init__(self, Parser.start, Parser.eof_check, Parser.automata)




# Waxeye Parser Generator
# www.waxeye.org
# Copyright (C) 2008-2010 Orlando Hill
# Licensed under the MIT license. See 'LICENSE' for details.

class Edge:
    def __init__(self, trans, state, voided):
        self.trans = trans
        self.state = state
        self.voided = voided


class State:
    def __init__(self, edges, match):
        self.edges = edges
        self.match = match


class FA:
    VOID = 0
    PRUNE = 1
    LEFT = 2
    POS = 3
    NEG = 4
    def __init__(self, type, states, mode):
        self.type = type
        self.states = states
        self.mode = mode


class ParseError:
    def __init__(self, pos, line, col, nt):
        self.pos = pos
        self.line = line
        self.col = col
        self.nt = nt

    def __str__(self):
        return "parse error: failed to match '%s' at line=%s, col=%s, pos=%s" % (self.nt, self.line, self.col, self.pos)


class AST:
    def __init__(self, type, children, pos):
        self.type = type
        self.children = children
        self.pos = pos

    def str_iter(self, ast, indent, acc):
        for i in range(0, indent[0] - 1):
            acc.append('    ')
        if indent[0] > 0:
            acc.append('->  ')
        acc.append(ast.type)
        indent[0] += 1
        for a in ast.children:
            acc.append('\n')
            if isinstance(a, AST):
                self.str_iter(a, indent, acc)
            else:
                for i in range(0, indent[0] - 1):
                    acc.append('    ')
                if indent[0] > 0:
                    acc.append('|   ')
                acc.append(a)
        indent[0] -= 1

    def __str__(self):
        acc = []
        self.str_iter(self, [0], acc)
        return ''.join(acc)


class WaxeyeParser:
    def __init__(self, start, eof_check, automata):
        self.start = start
        self.eof_check = eof_check
        self.automata = automata

    def parse(self, input):
        return WaxeyeParser.InnerParser(self.start, self.eof_check, self.automata, input).parse()

    class InnerParser:
        def __init__(self, start, eof_check, automata, input):
            self.start = start
            self.eof_check = eof_check
            self.automata = automata
            self.input = input
            self.input_len = len(input)
            self.input_pos = 0
            self.line = 1
            self.column = 0
            self.last_cr = False
            self.error_pos = 0
            self.error_line = 1
            self.error_col = 0
            self.error_nt = automata[start].type
            self.fa_stack = []
            self.cache = {}

        def parse(self):
            return self.do_eof_check(self.match_automaton(self.start))


        def match_automaton(self, index):
            start_pos = self.input_pos
            key = (index, start_pos)

            if self.cache.has_key(key):
                cachedItem = self.cache[key]
                self.restore_pos(cachedItem[1], cachedItem[2], cachedItem[3], cachedItem[4])
                return cachedItem[0]

            start_line = self.line
            start_col = self.column
            start_cr = self.last_cr
            automaton = self.automata[index]
            type = automaton.type
            mode = automaton.mode

            self.fa_stack = [automaton] + self.fa_stack
            res = self.match_state(0)
            self.fa_stack = self.fa_stack[1:]

            if mode == FA.POS:
                self.restore_pos(start_pos, start_line, start_col, start_cr)
                if res != False:
                    value = True
                else:
                    value = False
            elif mode == FA.NEG:
                self.restore_pos(start_pos, start_line, start_col, start_cr)
                if res != False:
                    value = self.update_error()
                else:
                    value = True
            elif res != False:
                if mode == FA.VOID:
                    value = True
                elif mode == FA.PRUNE:
                    l = len(res)
                    if l == 0:
                        value = True
                    elif l == 1:
                        value = res[0]
                    else:
                        value = AST(type, res, (start_pos, self.input_pos))
                else:
                    value = AST(type, res, (start_pos, self.input_pos))
            else:
                value = self.update_error()

            self.cache[key] = (value, self.input_pos, self.line, self.column, self.last_cr)
            return value


        # Returns a list of results so, need to check != False
        def match_state(self, index):
            state = self.fa_stack[0].states[index]
            res = self.match_edges(state.edges)
            if res != False:
                return res
            else:
                return state.match and []


        def match_edges(self, edges):
            if edges == []:
                return False
            else:
                res = self.match_edge(edges[0])
                if res != False:
                    return res
                else:
                    return self.match_edges(edges[1:])


        def match_edge(self, edge):
            start_pos = self.input_pos
            start_line = self.line
            start_col = self.column
            start_cr = self.last_cr
            t = edge.trans

            if t == -1: # use -1 for wild card
                if self.input_pos < self.input_len:
                    res = self.mv()
                else:
                    res = self.update_error()
            elif isinstance(t, str):
                if self.input_pos < self.input_len and t == self.input[self.input_pos]:
                    res = self.mv()
                else:
                    res = self.update_error()
            elif isinstance(t, list):
                if self.input_pos < self.input_len and self.within_set(t, ord(self.input[self.input_pos])):
                    res = self.mv()
                else:
                    res = self.update_error()
            elif isinstance(t, int):
                res = self.match_automaton(t)
            else:
                res = False

            if res:
                tran_res = self.match_state(edge.state)
                if tran_res != False:
                    if edge.voided or res == True:
                        return tran_res
                    else:
                        return [res] + tran_res
                else:
                    self.restore_pos(start_pos, start_line, start_col, start_cr)
                    return False
            else:
                return False


        def restore_pos(self, pos, line, col, cr):
            self.input_pos = pos
            self.line = line
            self.column = col
            self.last_cr = cr


        def update_error(self):
            if self.error_pos < self.input_pos:
                self.error_pos = self.input_pos
                self.error_line = self.line
                self.error_col = self.column
                self.error_nt = self.fa_stack[0].type
            return False


        def mv(self):
            ch = self.input[self.input_pos]
            self.input_pos += 1

            if ch == '\r':
                self.line += 1
                self.column = 0
                self.last_cr = True
            else:
                if ch == '\n':
                    if not self.last_cr:
                        self.line += 1
                        self.column = 0
                else:
                    self.column += 1
                self.last_cr = False

            return ch

        def do_eof_check(self, res):
            if res:
                if self.eof_check and self.input_pos < self.input_len:
                    # Create a parse error - Not all input consumed
                    return ParseError(self.error_pos, self.error_line, self.error_col, self.error_nt)
                else:
                    return res
            else:
                # Create a parse error
                return ParseError(self.error_pos, self.error_line, self.error_col, self.error_nt)


        # Takes a set and an ordinal
        def within_set(self, set, c):
            if set == []:
                return False
            else:
                aa = set[0]
                if isinstance(aa, str):
                    if ord(aa) == c:
                        return True
                    else:
                        if ord(aa) < c:
                            return self.within_set(set[1:], c)
                        else:
                            return False
                else:
                    # If not a String then must be a range (tuple)
                    if c >= aa[0] and c <= aa[1]:
                        return True
                    else:
                        if aa[1] < c:
                            return self.within_set(set[1:], c)
                        else:
                            return False



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement

import os
import sys

from distutils.core import setup
from distutils.core import Command
from subprocess import call
from os.path import join as pjoin
from pip.req import parse_requirements


if (sys.version_info < (2, 5, 0) or sys.version_info >= (3, 0, 0)):
    raise RuntimeError('Unsupported Python version. Supported versions are:' +
                       '2.5, 2.6, 2.7 / PyPy')


def read_version_string():
    version = None
    sys.path.insert(0, pjoin(os.getcwd()))
    from raxmon_cli import __version__
    version = __version__
    sys.path.pop(0)
    return version


class Pep8Command(Command):
    description = "run pep8 script"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            import pep8
            pep8
        except ImportError:
            print 'Missing "pep8" library. You can install it using pip: ' + \
                  'pip install pep8'
            sys.exit(1)

        cwd = os.getcwd()
        retcode = call(('pep8 %s/raxmon_cli/' %
                (cwd)).split(' '))
        sys.exit(retcode)


# Create a PipOptions object that stubs out options that older versions of pip expect to exist.
class PipOptions(object):
    def __init__(self):
        self.skip_requirements_regex = None
        self.default_vcs = None


class GenerateCompletionsCommand(Command):
    description = "Generate bash completions file for all the commands"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        path = pjoin(os.getcwd(), 'commands/')
        files = os.listdir(path)
        content = '\n'.join(['complete -F _optcomplete %s' % (f) for f in files])

        with open('contrib/completions.sh', 'w') as fp:
            fp.write(content)

        print('Done')

scripts = os.listdir(pjoin(os.getcwd(), 'commands/'))
scripts = [pjoin(os.getcwd(), 'commands/', path) for path in scripts]

pre_python26 = (sys.version_info[0] == 2 and sys.version_info[1] < 6)


options = PipOptions()
requires = [str(ir.req) for ir in parse_requirements('requirements.txt', options=options)]

if pre_python26:
    requires.append('simplejson')

setup(
    name='rackspace-monitoring-cli',
    version=read_version_string(),
    description='Command Line Utility for Rackspace Cloud Monitoring (MaaS)',
    author='Rackspace, Inc.',
    author_email='monitoring@rackspace.com',
    install_requires=requires,
    scripts=scripts,
    packages=[
        'raxmon_cli',
    ],
    package_dir={
        'raxmon_cli': 'raxmon_cli',
    },
    package_data={
        'raxmon_cli': ['data/cacert.pem'],
    },
    license='Apache License (2.0)',
    url='https://github.com/racker/rackspace-monitoring-cli',
    cmdclass={
        'pep8': Pep8Command,
        'gencompletions': GenerateCompletionsCommand
    },
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
